/*******************************************************************************
 * QMetry Automation Framework provides a powerful and versatile platform to author 
 * Automated Test Cases in Behavior Driven, Keyword Driven or Code Driven approach
 *                
 * Copyright 2016 Infostretch Corporation
 *
 * This program is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT
 * OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE
 *
 * You should have received a copy of the GNU General Public License along with this program in the name of LICENSE.txt in the root folder of the distribution. If not, see https://opensource.org/licenses/gpl-3.0.html
 *
 * See the NOTICE.TXT file in root folder of this source files distribution 
 * for additional information regarding copyright ownership and licenses
 * of other open source software / files used by QMetry Automation Framework.
 *
 * For any inquiry or need additional information, please contact support-qaf@infostretch.com
 *******************************************************************************/

package com.qmetry.qaf.automation.step.client.text;

import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;

/**
 * com.qmetry.qaf.automation.step.BaseTestStep.java
 * 
 * @author chirag.jayswal
 */
public abstract class BaseTestStep implements TestStep {
	protected String name;
	protected String description;
	transient protected Object[] actualArgs;
	protected String fileName;
	protected int lineNumber;
	protected int threshold;
	protected Map<String, Object> metaData = new HashMap<String, Object>();

	public BaseTestStep(String name, String description, Object... actualArgs) {
		this.name = name;
		this.description = description;
		setActualArgs(actualArgs);
	}

	protected BaseTestStep() {
		this("", "");
	}

	@Override
	public String getDescription() {
		return description;
	}

	@Override
	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	@Override
	public void setActualArgs(Object... args) {
		if (null != args) {
			actualArgs = Arrays.copyOf(args, args.length);
		}
	}

	@Override
	public void setDescription(String description) {
		this.description = description;
	}

	@Override
	public Object[] getActualArgs() {
		return actualArgs;
	}

	public void setFileName(String fileName) {
		this.fileName = fileName;
	}

	public void setLineNumber(int lineNumber) {
		this.lineNumber = lineNumber;
	}

	public void setThreshold(int threshold) {
		this.threshold = threshold;
	}

	@Override
	public Map<String, Object> getMetaData() {
		return metaData;
	}

	public void setMetaData(Map<String, Object> metaData) {
		this.metaData = metaData;
	}

	@Override
	public abstract TestStep clone();

}
