package com.qmetry.qaf.common;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.TreeSet;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.jdt.core.IAnnotation;
import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.jdt.core.IJavaElement;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.IMethod;
import org.eclipse.jdt.core.search.IJavaSearchConstants;
import org.eclipse.jdt.core.search.IJavaSearchScope;
import org.eclipse.jdt.core.search.SearchEngine;
import org.eclipse.jdt.core.search.SearchMatch;
import org.eclipse.jdt.core.search.SearchParticipant;
import org.eclipse.jdt.core.search.SearchPattern;
import org.eclipse.jdt.core.search.SearchRequestor;

import com.google.inject.Inject;
import com.google.inject.Singleton;
import com.qmetry.qaf.automation.step.client.text.BDDStepMatcherFactory;
import com.qmetry.qaf.automation.step.client.text.StringTestStep;

@Singleton
public class JavaStepHolder {

	@Inject
	private AbstractAnnotationDescriptor descriptor;

	private Map<ICompilationUnit, List<Entry>> cache = new HashMap<ICompilationUnit, List<Entry>>();

	protected IJavaSearchScope getScope(String filter) {
		if (filter == null)
			return SearchEngine.createWorkspaceScope();

		String[] names = filter.split(",");
		final List<IJavaElement> packages = new ArrayList<IJavaElement>();

		SearchPattern pattern = null;
		for (String name : names) {
			SearchPattern current = SearchPattern.createPattern(name.trim(), IJavaSearchConstants.PACKAGE,
					IJavaSearchConstants.ALL_OCCURRENCES, SearchPattern.R_EXACT_MATCH | SearchPattern.R_CASE_SENSITIVE);
			if (pattern == null) {
				pattern = current;
			} else {
				pattern = SearchPattern.createOrPattern(pattern, current);
			}
		}
		try {
			new SearchEngine().search(pattern, new SearchParticipant[] { SearchEngine.getDefaultSearchParticipant() },
					SearchEngine.createWorkspaceScope(), new SearchRequestor() {
						public void acceptSearchMatch(SearchMatch match) throws CoreException {
							packages.add((IJavaElement) match.getElement());
						}
					}, null);
		} catch (CoreException e) {
			e.printStackTrace();
		}
		return SearchEngine.createJavaSearchScope(packages.toArray(new IJavaElement[0]));
	}

	public void findMatches(final String stepCall, final Command command) {
		// long time = System.currentTimeMillis();
		if (!cache.isEmpty()) {
			for (List<Entry> entries : cache.values()) {
				for (Entry entry : entries) {
					if (BDDStepMatcherFactory.getStepMatcher(new StringTestStep(stepCall))
							.matches(entry.annotationValue, stepCall, new HashMap<String, Object>())) {
						command.match(entry.annotationValue, entry.method);
					}
				}
			}
			return;
		}

		// combine search patterns
		SearchPattern pattern = null;
		for (final String annotationName : descriptor.getNames()) {
			SearchPattern current = SearchPattern.createPattern(annotationName, IJavaSearchConstants.ANNOTATION_TYPE,
					IJavaSearchConstants.ANNOTATION_TYPE_REFERENCE,
					SearchPattern.R_EXACT_MATCH | SearchPattern.R_CASE_SENSITIVE);
			if (pattern == null) {
				pattern = current;
			} else {
				pattern = SearchPattern.createOrPattern(pattern, current);
			}
		}
		SearchPattern current = SearchPattern.createPattern("QAFTestStep", IJavaSearchConstants.ANNOTATION_TYPE,
				IJavaSearchConstants.ANNOTATION_TYPE_REFERENCE,
				SearchPattern.R_EXACT_MATCH | SearchPattern.R_CASE_SENSITIVE);
		if (pattern == null) {
			pattern = current;
		} else {
			pattern = SearchPattern.createOrPattern(pattern, current);
		}
		String[] STEPS = { "Given", "When", "Then", "And", "Using", "Having", "But", "QAFTestStep" };
		final List<String> listANnotations = Arrays.asList(STEPS);
		// execute search
		try {
			new SearchEngine().search(pattern, new SearchParticipant[] { SearchEngine.getDefaultSearchParticipant() },
					this.getScope(null), new SearchRequestor() {
						public void acceptSearchMatch(SearchMatch match) throws CoreException {
							if (match.getElement() instanceof IMethod) {
								IMethod method = (IMethod) match.getElement();
								IAnnotation[] annotations = method.getAnnotations();
								for (IAnnotation type : annotations) {
									// check annotation package
									if (listANnotations.contains(type.getElementName().substring(
											type.getElementName().lastIndexOf(".") + 1, type.getElementName().length()))
											// AbstractAnnotationDescriptor.checkPackage(type,
											// descriptor.getPackage())
											&& type.getMemberValuePairs() != null
											&& type.getMemberValuePairs().length > 0) {
										// verify pattern
										String annotationValue = "";
										for (int i = 0; i < type.getMemberValuePairs().length; i++) {
											if (type.getMemberValuePairs()[i].getMemberName().contains("description")
													|| type.getMemberValuePairs()[i].getMemberName().contains("value"))
												annotationValue = (String) type.getMemberValuePairs()[i].getValue();
										}
										List<Entry> entries = cache.get(method.getCompilationUnit());
										if (entries == null) {
											entries = new ArrayList<Entry>();
											cache.put(method.getCompilationUnit(), entries);
										}
										entries.add(new Entry(annotationValue, method));
										if (BDDStepMatcherFactory.getStepMatcher(new StringTestStep(stepCall))
												.matches(annotationValue, stepCall, new HashMap<String, Object>())) {
											command.match(annotationValue, method);
										}
									}
								}
							}
						}
					}, null);
		} catch (CoreException e) {
			e.printStackTrace();
		}
	}

	/**
	 * 
	 * @return This will return all the steps
	 */
	public Collection<String> findProposals() {

		if (cache.isEmpty()) {
			findMatches("", new Command() {
				@Override
				public void match(String annotationValue, IMethod method) {
				}
			});
		}

		Collection<String> proposals = new TreeSet<String>();
		for (List<Entry> entries : cache.values()) {
			for (Entry entry : entries) {
				proposals.add(entry.getAnnotationValue());
			}
		}
		return proposals;
	}

	public void initialize(ICompilationUnit element) {
		cache.clear();
		System.out.println("Clearing Cache as Compilation Unit Has Changed");
	}

	public static interface Command {
		void match(String annotationValue, IMethod method);
	}

	@SuppressWarnings("unused")
	private List<String> computeProposals(IJavaProject project, String fqn, int offset, final String content) {
		// ICompilationUnit compilationUnit;
		System.out.println("computeProposals");
		final List<String> abcd = new ArrayList<String>();
		SearchPattern current = SearchPattern.createPattern("QAFTestStep", IJavaSearchConstants.ANNOTATION_TYPE,
				IJavaSearchConstants.ANNOTATION_TYPE_REFERENCE,
				SearchPattern.R_EXACT_MATCH | SearchPattern.R_CASE_SENSITIVE);
		SearchPattern pattern = null;
		pattern = SearchPattern.createOrPattern(pattern, current);
		String[] STEPS = { "Given", "When", "Then", "And", "Using", "Having", "But", "QAFTestStep" };
		final List<String> listANnotations = Arrays.asList(STEPS);

		try {
			new SearchEngine().search(pattern, new SearchParticipant[] { SearchEngine.getDefaultSearchParticipant() },
					this.getScope(null), new SearchRequestor() {
						public void acceptSearchMatch(SearchMatch match) throws CoreException {
							if (match.getElement() instanceof IMethod) {
								IMethod method = (IMethod) match.getElement();
								IAnnotation[] annotations = method.getAnnotations();
								for (IAnnotation type : annotations) {
									// check annotation package
									if (listANnotations.contains(type.getElementName())
											&& type.getMemberValuePairs() != null
											&& type.getMemberValuePairs().length > 0) {

										// verify pattern
										String annotationValue = "";
										for (int i = 0; i < type.getMemberValuePairs().length; i++) {
											if (type.getMemberValuePairs()[i].getMemberName().contains("description")
													|| type.getMemberValuePairs()[i].getMemberName().contains("value"))
												annotationValue = (String) type.getMemberValuePairs()[i].getValue();
										}
										List<Entry> entries = cache.get(method.getCompilationUnit());
										if (entries == null) {
											entries = new ArrayList<Entry>();
											cache.put(method.getCompilationUnit(), entries);
										}
										entries.add(new Entry(annotationValue, method));
										if (BDDStepMatcherFactory.getStepMatcher(new StringTestStep(annotationValue))
												.matches(annotationValue, content, new HashMap<String, Object>())) {
											abcd.add(annotationValue);
										}
									}
								}
							}
						}
					}, null);
		} catch (CoreException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return abcd;
	}

	public static class Entry {
		private String annotationValue;
		private IMethod method;

		private Entry(String annotationValue, IMethod method) {
			this.annotationValue = annotationValue;
			this.method = method;
		}

		public String getAnnotationValue() {
			return annotationValue;
		}
	}
}
